<?php
/*
Plugin Name: VisionPay Payment Gateway
Description: Start accepting card payments today and experience the VisionPay difference!
Version: 1.0.9
Author: VisionPay
Plugin URI: https://visionpay.com.au/
License: GPLv2 or later
License URI: https://www.gnu.org/licenses/gpl-2.0.html
WC requires at least: 6.0
WC tested up to: 8.0
*/

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

add_action( 'before_woocommerce_init', function() {
    if ( class_exists( '\Automattic\WooCommerce\Utilities\FeaturesUtil' ) ) {
        \Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'custom_order_tables', __FILE__, true );
        \Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'cart_checkout_blocks', __FILE__, true );
    }
});

// Initialize payment gateway class

add_action('admin_enqueue_scripts', function($hook) {
    // Load only on WooCommerce settings pages
    if (isset($_GET['page']) && $_GET['page'] === 'wc-settings' && $_GET['section'] === 'visionpay') {
         wp_enqueue_script( 'wp-color-picker' );
        wp_enqueue_script(
            'visionpay-script',
            plugin_dir_url(__FILE__) . 'assets/js/visionpay-script.js',
            array('jquery'),
            '1.0',
            true
        );
    }
});


add_action( 'plugins_loaded', 'visionpay_init_gateway_class' );

function visionpay_init_gateway_class() {
    if ( class_exists( 'WC_Payment_Gateway' ) ) {
        class WC_Gateway_VisionPay extends WC_Payment_Gateway {

            public function __construct() {
                $this->id = 'visionpay';
                $this->has_fields = true;

                $this->method_title = __( 'VisionPay', 'visionpay' );
                $this->method_description = __( 'VisionPay Custom Payment Gateway for WooCommerce', 'visionpay' );

                // Initialize form fields and settings
                $this->init_form_fields();
                $this->init_settings();

                // User settings
                $this->title = $this->get_option( 'title' );
                $this->description = $this->get_option( 'description' );
                $this->enabled = $this->get_option( 'enabled' );
                $this->payment_mode = $this->get_option( 'payment_mode' );
                $this->btn_color = $this->get_option( 'btnColor' );
                $this->token_staging = $this->get_option( 'token_staging' );
                $this->token = $this->get_option( 'token' );

                // Save settings
                add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
            }

            public function init_form_fields() {
                $this->form_fields = array(
                    'enabled' => array(
                        'title'       => __( 'Enable/Disable', 'visionpay' ),
                        'label'       => __( 'Enable VisionPay', 'visionpay' ),
                        'type'        => 'checkbox',
                        'default'     => 'no'
                    ),
                    'title' => array(
                        'title'       => __( 'Title', 'visionpay' ),
                        'type'        => 'text',
                        'description' => __( 'This controls the title the user sees during checkout.', 'visionpay' ),
                        'default'     => __( 'VisionPay', 'visionpay' ),
                        'desc_tip'    => true,
                    ),
                    'description' => array(
                        'title'       => __( 'Description', 'visionpay' ),
                        'type'        => 'textarea',
                        'description' => __( 'This controls the description the user sees during checkout.', 'visionpay' ),
                        'default'     => __( 'Pay with VisionPay.', 'visionpay' ),
                    ),
                    'payment_mode' => array(
                        'title'       => __('Payment Mode', 'visionpayv'),
                        'type'        => 'select',
                        'description' => __('', 'visionpay'),
                        'options'     => array(
                            'production' => __('Production', 'visionpay'),
                            'staging' => __('Staging', 'visionpay')
                        ),
                        'default'     => 'option1',
                        'desc_tip'    => true,
                    ),
                    'token' => array(
                        'title'       => __( 'API Token Production' , 'visionpay' ),
                        'type'        => 'text',
                        'description' => __( 'Enter your VisionPay API token here.', 'visionpay' ),
                        'default'     => '',
                        'desc_tip'    => true,
                    ),
                    'token_staging' => array(
                        'title'       => __( 'API Token Staging' , 'visionpay' ),
                        'type'        => 'text',
                        'description' => __( 'Enter your VisionPay API token here.', 'visionpay' ),
                        'default'     => '',
                        'desc_tip'    => true,
                    ),
                    'btnColor' => array(
                        'title'       => __( 'Button Color', 'visionpay' ),
                        'type'        => 'text',
                        'description' => __( 'Specify the color of the payment button (e.g., "#FF5733").', 'visionpay' ),
                        'default'     => '#0c7bfa',
                        'desc_tip'    => true,
                        'class'       => 'custom-color-picker',
                    ),
                );
            }


            public function is_available() {
                $is_available = ('yes' === $this->enabled);
                if ( empty($this->token) ) {
                    $is_available = false;
                }
                return $is_available;
            }

            public function payment_fields() {
                if ( $this->description ) {
                    echo wpautop( wp_kses_post( $this->description ) );
                }

                // Get or create a pending order
                // $order_id = WC()->session->get('order_awaiting_payment');
                // if ( !$order_id ) {
                //     wc_add_notice( __( 'Payment error: Unable to generate order.', 'visionpay' ), 'error' );
                //     return;
                // }

                // $order = wc_get_order( $order_id );
                // if ( !$order ) {
                //     wc_add_notice( __( 'Payment error: Unable to retrieve order.', 'visionpay' ), 'error' );
                //     return;
                // }

                // Prepare the API endpoint and request body
                $payment_mode = $this->payment_mode;
                if($payment_mode === 'production'){
                    $api_url = 'https://apigateway.visionpay.com.au/api/ApiKey/authenticate';
                    $api_key = $this->token;
                    $host_url = 'https://apigateway.visionpay.com.au/HostedPage';
                }else {
                     $api_url = 'https://apigatewaystaging.visionpay.com.au/api/ApiKey/authenticate';
                    $api_key = $this->token_staging;
                    $host_url = 'https://apigatewaystaging.visionpay.com.au/HostedPage';
                }


                $body = array(
                    'accessToken' => $api_key
                );

                // Send POST request to obtain OAuth access token
                $response = wp_remote_post( $api_url, array(
                    'body'    => wp_json_encode( $body ),
                    'headers' => array(
                        'Content-Type' => 'application/json'
                    )
                ) );

                if ( is_wp_error( $response ) ) {
                    wc_add_notice( __( 'Payment error: Could not connect to VisionPay', 'visionpay' ), 'error' );
                    return;
                }

                $response_body = wp_remote_retrieve_body( $response );
                $response_data = json_decode( $response_body, true );

                if ( isset( $response_data['token'] ) && is_string( $response_data['token'] ) ) {
                    $token = $response_data['token'];
                    $reference = 'ORDER-' . uniqid();

                    $amount = 0;
                    // cart item
                    if (WC()->cart && WC()->cart->get_cart()) {
                        foreach (WC()->cart->get_cart() as $cart_item_key => $cart_item) {
                            $amount += $cart_item['line_total'];
                        }
                    }

                    // shipping fee
                    $chosen_shipping_methods = WC()->session->get('chosen_shipping_methods');
                    $packages = WC()->shipping()->get_packages();

                    foreach ($packages as $package_id => $package) {
                        foreach ($package['rates'] as $rate_id => $rate) {
                            if (in_array($rate_id, $chosen_shipping_methods)) {
                                $amount += $rate->cost;
                            }
                        }
                    }

                    $currency = 'AUD';
                    $return_url = home_url( '/checkout/' );
                    $denied_url = home_url( '/checkout/' );

                    echo '<iframe id="visionpayIframe" name="visionpayIframe" style="width: 100%; height: 950px; border: none;"></iframe>';
                    //echo '<iframe id="visionpayIframe" name="visionpayIframe" style="width: 100%; height: 950px; border: none;"></iframe><style>#visionpayIframe { display: none; } #visionpayIframe.show{ display: block; }</style>';

                    echo '<form id="visionpayForm" action="' . esc_url($host_url) . '" method="post" target="visionpayIframe">';
                    echo '<input type="hidden" name="token" value="' . esc_attr( $token ) . '">';
                    echo '<input type="hidden" name="reference" value="' . esc_attr( $reference ) . '">';
                    echo '<input type="hidden" name="amount" value="' . esc_attr( $amount ) . '">';
                    echo '<input type="hidden" name="currency" value="' . esc_attr( $currency ) . '">';
                    echo '<input type="hidden" name="returnUrl" value="' . esc_url( $return_url ) . '">';
                    echo '<input type="hidden" name="deniedUrl" value="' . esc_url( $denied_url ) . '">';
                    echo '<input type="hidden" name="showBrand" value="false">';
                    echo '<input type="hidden" name="btnColor" value="' . esc_attr( $this->btn_color ) . '">';
                    echo '<input type="hidden" name="expand" value="true">';
                    echo '<input type="hidden" name="showDetails" value="false">';
                    echo '</form>';
                    echo '<div id="visionpay-message" style="margin-top: 20px; font-weight: bold;"></div>';
                    ?>


                    <script type="text/javascript">
                      jQuery(document).ready(function(jQuery){

                        function validateFields() {

                          let isInvalid = false;
                          // Condition 1: Check for "woocommerce-invalid-required-field" class
                          jQuery("form.checkout.woocommerce-checkout .woocommerce-billing-fields .validate-required").each(function () {
                            jQuery(document).ajaxComplete(function(event, xhr, settings) {
                              if (settings.url.includes('wc-ajax=update_order_review')) {
                                if (jQuery(this).hasClass("woocommerce-validated")  == false || jQuery(this).hasClass("woocommerce-invalid") == true) {
                                  isInvalid = true;
                                }
                              }
                            });
                            if (jQuery(this).hasClass("woocommerce-invalid") == true) {
                              isInvalid = true;
                            }
                          });

                          // Condition 2: Check if any input fields are empty
                          jQuery("form.checkout.woocommerce-checkout .woocommerce-billing-fields .validate-required .input-text").each(function () {
                            if (jQuery(this).val() === "" || jQuery(this).val() === null) {
                              isInvalid = true;
                            }
                          });

                          // Condition 3: Check if any select fields have no value
                          jQuery("form.checkout.woocommerce-checkout .woocommerce-billing-fields .validate-required .state_select").each(function () {
                            if (jQuery(this).val() === "" || jQuery(this).val() === null) {
                              isInvalid = true;
                            }
                          });

                          // Display appropriate message or iframe
                          if (isInvalid) {
                            jQuery("#visionpay-message").text("To continue payment please fill in the information completely.");
                            jQuery("#visionpayIframe").removeClass("show");
                          } else {
                            jQuery("#visionpay-message").text("");
                            jQuery("#visionpayIframe").addClass("show");
                          }
                        }

                        var payment_method = jQuery("form.checkout.woocommerce-checkout input[name=\'payment_method\']:checked").val();

                        if (payment_method == "visionpay") {
                          setTimeout(function() {
                            // Validate fields when the page loads
                            validateFields();

                            // Validate fields dynamically when input or select fields change
                            jQuery("form.checkout.woocommerce-checkout .woocommerce-billing-fields .validate-required").on("input keyup change", function () {
                              validateFields();
                            });
                          }, 500);
                        }

                        jQuery("form.checkout.woocommerce-checkout").on("change", function () {
                          var payment_method = jQuery("form.checkout.woocommerce-checkout input[name='payment_method']:checked").val();
                          if (payment_method == "visionpay") {
                            setTimeout(function() {
                              // Validate fields when the page loads
                              validateFields();
                              // Validate fields dynamically when input or select fields change
                              jQuery("form.checkout.woocommerce-checkout .woocommerce-billing-fields .validate-required").on("input keyup change", function () {
                                validateFields();
                              });
                            }, 500);
                          }
                        });

                        document.getElementById("visionpayForm").submit();
                        if (!window.visionpayListenerAdded) {
                            window.addEventListener("message", function(event) {
                              if (event.data.type === "redirect") {
                                var formData = jQuery("form.checkout.woocommerce-checkout").serialize();
                                document.getElementById("visionpay-message").innerText = "Payment was successful!";
                                jQuery.ajax({
                                  url: "<?php echo admin_url('admin-ajax.php') ?>",
                                  type: "POST",
                                  data: {
                                    action: "visionpay_process_payment",
                                    form_data: formData
                                  },
                                  success: function(response) {
                                    if (response.success) {
                                      window.location.href = response.data.redirect;
                                    } else {
                                      document.getElementById("visionpay-message").innerText = "Payment error: " + response.data.message;
                                    }
                                  },
                                  error: function(jqXHR, textStatus, errorThrown) {
                                    document.getElementById("visionpay-message").innerText = "An error occurred while processing the payment.";
                                  }
                                });
                              } else if (event.data.type === "defined") {
                                document.getElementById("visionpay-message").innerText = "Payment failed. Please try again.";
                              }
                            });
                            window.visionpayListenerAdded = true; // Flag to indicate the listener has been added
                        }
                      });
                    </script>
                    <?php
                } else {
                    wc_add_notice( __( 'Payment error: Invalid response from VisionPay', 'visionpay' ), 'error' );
                    return;
                }
            }
        }
    }
}

// Add VisionPay to WooCommerce payment gateways
add_filter( 'woocommerce_payment_gateways', 'add_visionpay_gateway_class' );

function add_visionpay_gateway_class( $gateways ) {
    $gateways[] = 'WC_Gateway_VisionPay';
    return $gateways;
}

// AJAX handler to process the payment
add_action( 'wp_ajax_visionpay_process_payment', 'visionpay_process_payment' );
add_action( 'wp_ajax_nopriv_visionpay_process_payment', 'visionpay_process_payment' );

function visionpay_process_payment() {

    if (!isset($_POST['form_data'])) {
        wp_send_json_error('No data received');
    }

    // Parse serialized data
    parse_str($_POST['form_data'], $form_data);
    // var_dump($form_data["billing_first_name"]);

    $address = array(
        'first_name' => $form_data['billing_first_name'],
        'last_name' => $form_data['billing_last_name'],
        'company' => $form_data['billing_company'],
        'email' => $form_data['billing_email'],
        'phone' => $form_data['billing_phone'],
        'address_1' => $form_data['billing_address_1'],
        'address_2' => $form_data['billing_address_2'],
        'city' => $form_data['billing_city'],
        'state' => $form_data['billing_state'],
        'postcode' => $form_data['billing_postcode'],
        'country' => $form_data['billing_country']
    );

    $order = wc_create_order();

    foreach ( WC()->cart->get_cart() as $cart_item_key => $cart_item ) {
        $order->add_product( $cart_item['data'], $cart_item['quantity'] );
    }


    // apply coupon
    foreach (WC()->cart->get_coupons() as $code => $coupon) :
        $order->apply_coupon($code);
    endforeach;

    // Get shipping method from form data
    $shipping_method = $form_data['shipping_method']; // e.g., 'flat_rate:8'

    // Extract method_id and instance_id from shipping_method (e.g., flat_rate:8)
    // list($method_id, $instance_id) = explode(':', $shipping_method);
    $method_id = "";
    $instance_id = "";
    foreach ($shipping_method as $item_method) {
      // Split the string into $method_id and $instance_id
      if (strpos($item_method, ':') !== false) {
        list($method_id, $instance_id) = explode(':', $item_method);
      }
    }


    // Get the shipping zones to find the correct method
    $shipping_zones = WC_Shipping_Zones::get_zones();

    $shipping_item = new WC_Order_Item_Shipping();

    // Flag to check if the shipping method was found
    $shipping_found = false;

    // Loop through shipping packages from WC_Session (They can be multiple in some cases)
    foreach ( WC()->cart->get_shipping_packages() as $package_id => $package ) {
        // Check if a shipping for the current package exist
      if ( WC()->session->__isset( 'shipping_for_package_'.$package_id ) ) {
            // Loop through shipping rates for the current package
        foreach ( WC()->session->get( 'shipping_for_package_'.$package_id )['rates'] as $shipping_rate_id => $shipping_rate ) {
          $wc_rate_id     = $shipping_rate->get_id();
          $wc_method_id   = $shipping_rate->get_method_id();
          $wc_instance_id = $shipping_rate->get_instance_id();
          $wc_label_name  = $shipping_rate->get_label();
          $wc_cost        = $shipping_rate->get_cost();
          $wc_tax_cost    = $shipping_rate->get_shipping_tax();
          $wc_taxes       = $shipping_rate->get_taxes();
          if ($wc_method_id === $method_id && $wc_instance_id == $instance_id) {
            // Add this information to the order as a shipping item
            $shipping_item = new WC_Order_Item_Shipping();
            $shipping_item->set_method_title( $wc_label_name );
            $shipping_item->set_method_id( $wc_method_id );
            $shipping_item->set_total( $wc_cost );

            $order->add_item( $shipping_item );
          }

        }
      }
    }

    $order->set_address($address, 'billing');
    $order->set_address($address, 'shipping');
    $order->calculate_totals();
    $order->save();
    $order->update_status( 'pending', 'Order created pending payment' );

    if ( $order ) {
        if ( is_user_logged_in() ) {
            $user_id = get_current_user_id();
            $order->set_customer_id( $user_id );
        }
        $order->update_status( 'processing', __( 'Payment successfully completed via VisionPay', 'visionpay' ) );
        wc_reduce_stock_levels( $order );
        WC()->cart->empty_cart();

        wp_send_json_success( array( 'redirect' => $order->get_checkout_order_received_url() ) );
    } else {
        WC()->cart->empty_cart();
        wp_send_json_error( array( 'message' => 'Order not found' ) );
    }
}


// Create pending order before customer details in checkout
//add_action('woocommerce_checkout_before_customer_details', 'visionpay_create_pending_order');

function visionpay_create_pending_order() {
    if ( ! WC()->session->get( 'order_awaiting_payment' ) ) {
        $cart = WC()->cart;
        if ( $cart && ! empty( $cart->get_cart() ) ) {
            $order = wc_create_order();

            foreach ( $cart->get_cart() as $cart_item_key => $cart_item ) {
                $order->add_product( $cart_item['data'], $cart_item['quantity'] );
            }

            if ( is_user_logged_in() ) {
                $user_id = get_current_user_id();
                $order->set_customer_id( $user_id );
            }

            $order->set_address( WC()->customer->get_billing(), 'billing' );
            $order->set_address( WC()->customer->get_shipping(), 'shipping' );

            $order->set_customer_ip_address( WC_Geolocation::get_ip_address() );

            $order->calculate_totals();
            $order->update_status( 'pending', 'Order created pending payment' );

            WC()->session->set( 'order_awaiting_payment', $order->get_id() );
        }
    }
}